// SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
// SPDX-FileCopyrightText: 2020 Harald Sitter <sitter@kde.org>

#include <KPluginFactory>

#include "module.h"

K_PLUGIN_CLASS_WITH_JSON(Module, "smart.json")

#include "main.moc"
